@@ -2,8 +2,8 @@ use crate::header::{check_header_value, get_header};
 use crate::http_version::get_http_version;
 use crate::method::get_method;
 // use crate::parse::ParsePosition::Headers;
-use crate::request::{RequestError, RequestPart};
-use crate::{HttpVersion, Method, RequestUri, Headers};
+use crate::request::{RequestBuilder, RequestError};
+use crate::{Headers, Method, RequestUri};
 use std::error::Error;
 
 #[derive(Debug, PartialEq)]
@@ -16,10 +16,11 @@ pub enum ParsePosition {
 
 #[inline]
 fn parse_request_line(
+    current_position: &mut ParsePosition,
     stop_position: &mut ParsePosition,
     buffer: &mut Vec<u8>,
     index: &mut usize,
-) -> Result<RequestPart, Box<dyn Error>> {
+) -> Result<RequestBuilder, Box<dyn Error>> {
     // meta[0] - sp1
     // meta[1] - sp2
     // meta[2] - request line end
@@ -48,7 +49,7 @@ fn parse_request_line(
     }
 
     if meta[2] == 0 {
-        return Ok(RequestPart::empty());
+        return Ok(RequestBuilder::empty());
     }
 
     if !(meta[0] > *index && meta[1] > meta[0] && meta[2] > meta[1]) {
@@ -59,6 +60,8 @@ fn parse_request_line(
     let http_version = get_http_version(&buffer[(meta[1] + 1)..=meta[2]])?;
     let request_uri = RequestUri::new(buffer[(meta[0] + 1)..meta[1]].to_owned());
 
+    *current_position = ParsePosition::Headers;
+
     *stop_position = match method {
         Method::Get => ParsePosition::Body,
         Method::Post => ParsePosition::RequestEnd,
@@ -67,7 +70,7 @@ fn parse_request_line(
 
     *index += meta[2] + 3;
 
-    Ok(RequestPart {
+    Ok(RequestBuilder {
         method: Some(method),
         request_uri: Some(request_uri),
         http_version: Some(http_version),
@@ -78,7 +81,7 @@ fn parse_request_line(
 
 #[test]
 fn parse_request_line_0() {
-    let reference = RequestPart {
+    let reference = RequestBuilder {
         method: Some(Method::Get),
         request_uri: Some(RequestUri::new(b"/path".to_vec())),
         http_version: Some(HttpVersion::Http11),
@@ -86,18 +89,33 @@ fn parse_request_line_0() {
         body: None,
     };
 
+    let mut current_position = ParsePosition::RequestLine;
     let mut stop_position = ParsePosition::RequestEnd;
     let mut index = 0usize;
     let mut buffer = b"GET /path HTTP/1.1\r\n".to_vec();
-    let request_part = parse_request_line(&mut stop_position, &mut buffer, &mut index).unwrap();
+    let request_part = parse_request_line(
+        &mut current_position,
+        &mut stop_position,
+        &mut buffer,
+        &mut index,
+    )
+    .unwrap();
     assert_eq!(request_part, reference);
 }
 
+#[inline]
+fn check_min_header_length(s_idx: usize, e_idx: usize) -> Result<(), Box<dyn Error>> {
+    if e_idx - s_idx < 5 {
+        return Err(RequestError::InvalidHeader.into());
+    }
+    Ok(())
+}
+
 fn parse_headers(
-    stop_position: &mut ParsePosition,
+    current_position: &mut ParsePosition,
     buffer: &mut Vec<u8>,
     index: &mut usize,
-) -> Result<RequestPart, Box<dyn Error>> {
+) -> Result<RequestBuilder, Box<dyn Error>> {
     let mut index_pairs = Vec::<(usize, usize)>::new();
 
     let mut s_idx = *index;
@@ -105,15 +123,22 @@ fn parse_headers(
 
     for i in *index..buffer.len() {
         if buffer[i] == lf!() {
-            if buffer[i - 1] == cr!() {
-                e_idx = i - 2;
-                index_pairs.push((s_idx, e_idx));
-                s_idx = i + 1;
-                e_idx = i + 1;
+            if buffer[i - 1] != cr!() {
+                return Err(RequestError::InvalidHeader.into());
             }
+            if buffer[i - 2] == lf!() {
+                break;
+            }
+            e_idx = i - 2;
+            check_min_header_length(s_idx, e_idx)?;
+            index_pairs.push((s_idx, e_idx));
+            s_idx = i + 1;
+            e_idx = i + 1;
         }
     }
 
+    println!("{:?}", index_pairs);
+
     let mut headers = Headers::new();
 
     for (s_idx, e_idx) in index_pairs {
@@ -134,9 +159,12 @@ fn parse_headers(
         headers.insert(header_enum, header_value.to_vec());
     }
 
-    // idx = last_index + 3;
+    if *index - e_idx > 0 && buffer[e_idx] == lf!() && buffer[e_idx + 1] == cr!() {
+        *current_position = ParsePosition::Body;
+        *index = e_idx + 2;
+    }
 
-    Ok(RequestPart {
+    Ok(RequestBuilder {
         method: None,
         request_uri: None,
         http_version: None,
@@ -148,34 +176,22 @@ fn parse_headers(
 pub fn parse_request(
     current_position: &mut ParsePosition,
     stop_position: &mut ParsePosition,
-    vec: &mut Vec<u8>,
-) -> Result<RequestPart, Box<dyn Error>> {
-    let mut idx = 0;
+    buffer: &mut Vec<u8>,
+) -> Result<RequestBuilder, Box<dyn Error>> {
+    let mut index = 0;
 
-    let mut method = None;
-    let mut request_uri = None;
-    let mut http_version = None;
-    let mut headers = None;
-    let mut body = None;
-
-    let mut request_part = RequestPart::empty();
+    let mut request_part = RequestBuilder::empty();
 
     if *current_position == ParsePosition::RequestLine {
-        request_part = request_part + parse_request_line(stop_position, vec, &mut idx)?;
+        request_part =
+            request_part + parse_request_line(current_position, stop_position, buffer, &mut index)?;
     }
 
     if *current_position == ParsePosition::Headers {
-        request_part = request_part + parse_headers(stop_position, vec, &mut idx)?;
+        request_part = request_part + parse_headers(current_position, buffer, &mut index)?;
     }
 
-    *vec = vec.split_off(idx);
-    idx = 0;
+    *buffer = buffer.split_off(index);
 
-    Ok(RequestPart {
-        method,
-        request_uri,
-        http_version,
-        headers,
-        body,
-    })
+    Ok(request_part)
 }
